@@ -16,93 +16,59 @@
 namespace v8 {
 namespace internal {
 
-void Scavenger::PromotionList::View::PushRegularObject(HeapObject object,
-                                                       int size) {
-  promotion_list_->PushRegularObject(task_id_, object, size);
+void Scavenger::PromotionList::Local::PushRegularObject(HeapObject object,
+                                                        int size) {
+  regular_object_promotion_list_local_.Push({object, size});
 }
 
-void Scavenger::PromotionList::View::PushLargeObject(HeapObject object, Map map,
-                                                     int size) {
-  promotion_list_->PushLargeObject(task_id_, object, map, size);
+void Scavenger::PromotionList::Local::PushLargeObject(HeapObject object,
+                                                      Map map, int size) {
+  large_object_promotion_list_local_.Push({object, map, size});
 }
 
-bool Scavenger::PromotionList::View::IsEmpty() {
-  return promotion_list_->IsEmpty();
+size_t Scavenger::PromotionList::Local::LocalPushSegmentSize() const {
+  return regular_object_promotion_list_local_.PushSegmentSize() +
+         large_object_promotion_list_local_.PushSegmentSize();
 }
 
-size_t Scavenger::PromotionList::View::LocalPushSegmentSize() {
-  return promotion_list_->LocalPushSegmentSize(task_id_);
-}
-
-bool Scavenger::PromotionList::View::Pop(struct PromotionListEntry* entry) {
-  return promotion_list_->Pop(task_id_, entry);
-}
-
-void Scavenger::PromotionList::View::FlushToGlobal() {
-  promotion_list_->FlushToGlobal(task_id_);
-}
-
-bool Scavenger::PromotionList::View::IsGlobalPoolEmpty() {
-  return promotion_list_->IsGlobalPoolEmpty();
-}
-
-bool Scavenger::PromotionList::View::ShouldEagerlyProcessPromotionList() {
-  return promotion_list_->ShouldEagerlyProcessPromotionList(task_id_);
-}
-
-void Scavenger::PromotionList::PushRegularObject(int task_id, HeapObject object,
-                                                 int size) {
-  regular_object_promotion_list_.Push(task_id, ObjectAndSize(object, size));
-}
-
-void Scavenger::PromotionList::PushLargeObject(int task_id, HeapObject object,
-                                               Map map, int size) {
-  large_object_promotion_list_.Push(task_id, {object, map, size});
-}
-
-bool Scavenger::PromotionList::IsEmpty() {
-  return regular_object_promotion_list_.IsEmpty() &&
-         large_object_promotion_list_.IsEmpty();
-}
-
-size_t Scavenger::PromotionList::LocalPushSegmentSize(int task_id) {
-  return regular_object_promotion_list_.LocalPushSegmentSize(task_id) +
-         large_object_promotion_list_.LocalPushSegmentSize(task_id);
-}
-
-bool Scavenger::PromotionList::Pop(int task_id,
-                                   struct PromotionListEntry* entry) {
+bool Scavenger::PromotionList::Local::Pop(struct PromotionListEntry* entry) {
   ObjectAndSize regular_object;
-  if (regular_object_promotion_list_.Pop(task_id, &regular_object)) {
+  if (regular_object_promotion_list_local_.Pop(&regular_object)) {
     entry->heap_object = regular_object.first;
     entry->size = regular_object.second;
     entry->map = entry->heap_object.map();
     return true;
   }
-  return large_object_promotion_list_.Pop(task_id, entry);
+  return large_object_promotion_list_local_.Pop(entry);
 }
 
-void Scavenger::PromotionList::FlushToGlobal(int task_id) {
-  regular_object_promotion_list_.FlushToGlobal(task_id);
-  large_object_promotion_list_.FlushToGlobal(task_id);
+void Scavenger::PromotionList::Local::Publish() {
+  regular_object_promotion_list_local_.Publish();
+  large_object_promotion_list_local_.Publish();
 }
 
-size_t Scavenger::PromotionList::GlobalPoolSize() const {
-  return regular_object_promotion_list_.GlobalPoolSize() +
-         large_object_promotion_list_.GlobalPoolSize();
+bool Scavenger::PromotionList::Local::IsGlobalPoolEmpty() const {
+  return regular_object_promotion_list_local_.IsGlobalEmpty() &&
+         large_object_promotion_list_local_.IsGlobalEmpty();
 }
 
-bool Scavenger::PromotionList::IsGlobalPoolEmpty() {
-  return regular_object_promotion_list_.IsGlobalPoolEmpty() &&
-         large_object_promotion_list_.IsGlobalPoolEmpty();
-}
-
-bool Scavenger::PromotionList::ShouldEagerlyProcessPromotionList(int task_id) {
+bool Scavenger::PromotionList::Local::ShouldEagerlyProcessPromotionList()
+    const {
   // Threshold when to prioritize processing of the promotion list. Right
   // now we only look into the regular object list.
   const int kProcessPromotionListThreshold =
       kRegularObjectPromotionListSegmentSize / 2;
-  return LocalPushSegmentSize(task_id) < kProcessPromotionListThreshold;
+  return LocalPushSegmentSize() < kProcessPromotionListThreshold;
+}
+
+bool Scavenger::PromotionList::IsEmpty() const {
+  return regular_object_promotion_list_.IsEmpty() &&
+         large_object_promotion_list_.IsEmpty();
+}
+
+size_t Scavenger::PromotionList::Size() const {
+  return regular_object_promotion_list_.Size() +
+         large_object_promotion_list_.Size();
 }
 
 void Scavenger::PageMemoryFence(MaybeObject object) {
@@ -169,7 +135,7 @@ CopyAndForwardResult Scavenger::SemiSpaceCopyObject(
     }
     HeapObjectReference::Update(slot, target);
     if (object_fields == ObjectFields::kMaybePointers) {
-      copied_list_.Push(ObjectAndSize(target, object_size));
+      copied_list_local_.Push(ObjectAndSize(target, object_size));
     }
     copied_size_ += object_size;
     return CopyAndForwardResult::SUCCESS_YOUNG_GENERATION;
@@ -217,7 +183,7 @@ CopyAndForwardResult Scavenger::PromoteObject(Map map, THeapObjectSlot slot,
     }
     HeapObjectReference::Update(slot, target);
     if (object_fields == ObjectFields::kMaybePointers) {
-      promotion_list_.PushRegularObject(target, object_size);
+      promotion_list_local_.PushRegularObject(target, object_size);
     }
     promoted_size_ += object_size;
     return CopyAndForwardResult::SUCCESS_OLD_GENERATION;
@@ -246,7 +212,7 @@ bool Scavenger::HandleLargeObject(Map map, HeapObject object, int object_size,
       surviving_new_large_objects_.insert({object, map});
       promoted_size_ += object_size;
       if (object_fields == ObjectFields::kMaybePointers) {
-        promotion_list_.PushLargeObject(object, map, object_size);
+        promotion_list_local_.PushLargeObject(object, map, object_size);
       }
     }
     return true;
