@@ -14,6 +14,7 @@
 #include "src/objects/api-callbacks-inl.h"
 #include "src/objects/arguments-inl.h"
 #include "src/objects/bigint-inl.h"
+#include "src/objects/call-site-info-inl.h"
 #include "src/objects/cell-inl.h"
 #include "src/objects/code-inl.h"
 #include "src/objects/compilation-cache-table-inl.h"
@@ -72,7 +73,6 @@
 #include "src/objects/shared-function-info-inl.h"
 #include "src/objects/slots-atomic-inl.h"
 #include "src/objects/slots-inl.h"
-#include "src/objects/stack-frame-info-inl.h"
 #include "src/objects/string-inl.h"
 #include "src/objects/string-set-inl.h"
 #include "src/objects/string-table-inl.h"
