@@ -661,6 +661,10 @@ bool V8_EXPORT_PRIVATE IsJSCompatibleSignature(const FunctionSig* sig,
   V(I64AtomicCompareExchange16U, 0xfe4d, l_ill) \
   V(I64AtomicCompareExchange32U, 0xfe4e, l_ill)
 
+#define FOREACH_ATOMIC_0_OPERAND_OPCODE(V)                      \
+  /* AtomicFence does not target a particular linear memory. */ \
+  V(AtomicFence, 0xfe03, v_v)
+
 #define FOREACH_GC_OPCODE(V)                                         \
   V(StructNewWithRtt, 0xfb01, _)                                     \
   V(StructNewDefaultWithRtt, 0xfb02, _)                              \
@@ -699,19 +703,19 @@ bool V8_EXPORT_PRIVATE IsJSCompatibleSignature(const FunctionSig* sig,
   V(RefIsFunc, 0xfb50, _)                                            \
   V(RefIsData, 0xfb51, _)                                            \
   V(RefIsI31, 0xfb52, _)                                             \
+  V(RefIsArray, 0xfb53, _) /* not standardized - V8 experimental */  \
   V(RefAsFunc, 0xfb58, _)                                            \
   V(RefAsData, 0xfb59, _)                                            \
   V(RefAsI31, 0xfb5a, _)                                             \
+  V(RefAsArray, 0xfb5b, _) /* not standardized - V8 experimental */  \
   V(BrOnFunc, 0xfb60, _)                                             \
   V(BrOnData, 0xfb61, _)                                             \
   V(BrOnI31, 0xfb62, _)                                              \
+  V(BrOnArray, 0xfb66, _) /* not standardized - V8 experimental */   \
   V(BrOnNonFunc, 0xfb63, _)                                          \
   V(BrOnNonData, 0xfb64, _)                                          \
-  V(BrOnNonI31, 0xfb65, _)
-
-#define FOREACH_ATOMIC_0_OPERAND_OPCODE(V)                      \
-  /* AtomicFence does not target a particular linear memory. */ \
-  V(AtomicFence, 0xfe03, v_v)
+  V(BrOnNonI31, 0xfb65, _)                                           \
+  V(BrOnNonArray, 0xfb67, _) /* not standardized - V8 experimental */
 
 // All opcodes.
 #define FOREACH_OPCODE(V)            \
